/*   VLC-REMOTE for MAEMO 5
*   Copyright (C) 2010 Schutz Sacha <istdasklar@gmail.com>, Dru Moore <usr@dru-id.co.uk>, Yann Nave <yannux@onbebop.net>
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License version 2,
*   or (at your option) any later version, as published by the Free
*   Software Foundation
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details
*
*   You should have received a copy of the GNU General Public
*   License along with this program; if not, write to the
*   Free Software Foundation, Inc.,
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#ifndef PLAYERMAINWINDOW_H
#define PLAYERMAINWINDOW_H

#include <QMainWindow>
#include <QtNetwork>
#include <QtXml>
#include <QTimer>
#include "playlistmainwindow.h"
#include "browsemainwindow.h"
#include "favouritesmainwindow.h"
#include "vlcstatus.h"
#if defined(Q_WS_S60) || defined(Q_WS_MAEMO_5)
  #include <mce/dbus-names.h>
  #include <mce/mode-names.h>
  #include <QDBusConnection>
  #include <QDBusMessage>
  #include <QDBusInterface>
#endif

namespace Ui {
    class PlayerMainWindow;
}

class PlayerMainWindow : public QMainWindow {
    Q_OBJECT
public:
    explicit PlayerMainWindow(QWidget *parent = 0);
    ~PlayerMainWindow();

public slots:
    void displayStateChanged(const QDBusMessage &message);
    void askStatus();
    void showSettings();
    void showConfig();
    void showAbout();
    void showFavourites();
    void play();
    void playpause();
    void stop();
    void pause();
    void previous();
    void next();
    void fullscreen();
    void volUp();
    void volDown();
    void volMute();
    void slide(int value);
    void updateUiWithCurrentStatus();
    void setCoverArt(const QByteArray data);
    void setCoverArtFromPixmap(QPixmap image);
    void getCoverArt(int id);
    void finished(QNetworkReply * reply);
    void error(QNetworkReply::NetworkError code);
    void readReady();
    void updateFromSettings();
    void orientationChanged();
    void setPortrait();
    void setLandscape();
    void setAutoRotate();
    void playlistIdUpdated(int id, bool hasart, QString extension);


protected slots:
    void parseXmlStatus();

protected:
    void changeEvent(QEvent *e);
    void closeEvent(QCloseEvent *);


private:
    Ui::PlayerMainWindow *ui;
    PlayListMainWindow * mPlayListMainWindow;
    BrowseMainWindow * mBrowserMainWindow;
    FavouritesMainWindow * mFavouritesMainWindow;
    QNetworkAccessManager * mNetManager;
    QString mIp;
    QTimer * mTimer;
    int mVolume;
    int mMuted;
    VlcStatus mCurrentStatus;
    QByteArray mResponse;
    bool mIsLandscape;
    bool mHasImage;
    bool mIsFirstStatusCall;
    static QDBusConnection dBusConnection;
    static QDBusInterface* dBusInterface;
};

#endif // PLAYERMAINWINDOW_H
